@@ -19,7 +19,9 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  * 02110-1301, USA.
  */
+
 #define _GNU_SOURCE
+
 #include "config.h"
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -171,7 +173,7 @@ static int data_handler(void *user, const char *section, const char *name,
                 option.local_echo = false;
             }
         }
-        else if (!strcmp(name, "hex-mode"))
+        else if (!strcmp(name, "hexadecimal"))
         {
             if (!strcmp(value, "enable"))
             {
@@ -318,6 +320,11 @@ void config_file_parse(void)
     int ret;
 
     c = malloc(sizeof(struct config_t));
+    if (!c)
+    {
+        fprintf(stderr, "Error: Insufficient memory allocation");
+        exit(EXIT_FAILURE);
+    } 
     memset(c, 0, sizeof(struct config_t));
 
     // Find config file
