@@ -68,7 +68,9 @@ struct option_t option =
 
 void print_help(char *argv[])
 {
-    printf("Usage: %s [<options>] <tty-device|config>\n", argv[0]);
+    printf("Usage: %s [<options>] <tty-device|sub-config>\n", argv[0]);
+    printf("\n");
+    printf("Connect to tty-device directly or via sub-configuration.\n");
     printf("\n");
     printf("Options:\n");
     printf("  -b, --baudrate <bps>             Baud rate (default: 115200)\n");
@@ -88,11 +90,11 @@ void print_help(char *argv[])
     printf("  -m, --map <flags>                Map special characters\n");
     printf("  -c, --color 0..255|none|list     Colorize tio text (default: 15)\n");
     printf("  -S, --socket <socket>            Redirect I/O to socket\n");
-    printf("  -x, --hex-mode                   Enable hexadecimal mode\n");
+    printf("  -x, --hexadecimal                Enable hexadecimal mode\n");
     printf("  -v, --version                    Display version\n");
     printf("  -h, --help                       Display help\n");
     printf("\n");
-    printf("Options may be set via configuration file.\n");
+    printf("Options and sub-configurations may be set via configuration file.\n");
     printf("\n");
     printf("In session, press ctrl-t q to quit.\n");
     printf("\n");
@@ -199,7 +201,7 @@ void options_parse(int argc, char *argv[])
             {"socket",           required_argument, 0, 'S'                  },
             {"map",              required_argument, 0, 'm'                  },
             {"color",            required_argument, 0, 'c'                  },
-            {"hex-mode",         no_argument,       0, 'x'                  },
+            {"hexadecimal",      no_argument,       0, 'x'                  },
             {"version",          no_argument,       0, 'v'                  },
             {"help",             no_argument,       0, 'h'                  },
             {0,                  0,                 0,  0                   }
