@@ -78,15 +78,21 @@ static bool map_o_nl_crnl = false;
 static bool map_o_del_bs = false;
 static char hex_chars[2];
 static unsigned char hex_char_index = 0;
+static char tty_buffer[BUFSIZ*2];
+static size_t tty_buffer_count = 0;
+static char *tty_buffer_write_ptr = tty_buffer;
 
 static void optional_local_echo(char c)
 {
     if (!option.local_echo)
+    {
         return;
+    }
     print(c);
-    fflush(stdout);
     if (option.log)
-        log_write(c);
+    {
+        log_putc(c);
+    }
 }
 
 inline static bool is_valid_hex(char c)
@@ -96,17 +102,85 @@ inline static bool is_valid_hex(char c)
 
 inline static unsigned char char_to_nibble(char c)
 {
-    if(c >= '0' && c <= '9'){
+    if(c >= '0' && c <= '9')
+    {
         return c - '0';
-    } else if(c >= 'a' && c <= 'f'){
+    }
+    else if (c >= 'a' && c <= 'f')
+    {
         return c - 'a' + 10;
-    } else if(c >= 'A' && c <= 'F'){
+    }
+    else if (c >= 'A' && c <= 'F')
+    {
         return c - 'A' + 10;
-    } else {
+    }
+    else
+    {
         return 0;
     }
 }
 
+void tty_flush(int fd)
+{
+    ssize_t count;
+
+    do
+    {
+        count = write(fd, tty_buffer, tty_buffer_count);
+        if (count < 0)
+        {
+            // Error
+            debug_printf("Write error while flushing tty buffer (%s)", strerror(errno));
+            break;
+        }
+        tty_buffer_count -= count;
+    }
+    while (tty_buffer_count > 0);
+
+    // Reset
+    tty_buffer_write_ptr = tty_buffer;
+    tty_buffer_count = 0;
+}
+
+ssize_t tty_write(int fd, const void *buffer, size_t count)
+{
+    ssize_t bytes_written = 0;
+
+    if (option.output_delay)
+    {
+        // Write byte by byte with output delay
+        for (size_t i=0; i<count; i++)
+        {
+            ssize_t retval = write(fd, buffer, 1);
+            if (retval < 0)
+            {
+                // Error
+                debug_printf("Write error (%s)", strerror(errno));
+                break;
+            }
+            bytes_written += retval;
+            fsync(fd);
+            delay(option.output_delay);
+        }
+    }
+    else
+    {
+        // Flush tty buffer if too full
+        if ((tty_buffer_count + count) > BUFSIZ)
+        {
+            tty_flush(fd);
+        }
+
+        // Copy bytes to tty write buffer
+        memcpy(tty_buffer_write_ptr, buffer, count);
+        tty_buffer_write_ptr += count;
+        tty_buffer_count += count;
+        bytes_written = count;
+    }
+
+    return bytes_written;
+}
+
 static void output_hex(char c)
 {
     hex_chars[hex_char_index++] = c;
@@ -118,16 +192,15 @@ static void output_hex(char c)
 
         optional_local_echo(hex_value);
 
-        ssize_t status = write(fd, &hex_value, 1);
+        ssize_t status = tty_write(fd, &hex_value, 1);
         if (status < 0)
         {
             warning_printf("Could not write to tty device");
-        } else
+        }
+        else
         {
             tx_total++;
         }
-
-        fsync(fd);
     }
 }
 
@@ -251,7 +324,6 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
             case KEY_L:
                 /* Clear screen using ANSI/VT100 escape code */
                 printf("\033c");
-                fflush(stdout);
                 break;
 
             case KEY_Q:
@@ -304,6 +376,11 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
     }
 }
 
+void stdin_restore(void)
+{
+    tcsetattr(STDIN_FILENO, TCSANOW, &stdin_old);
+}
+
 void stdin_configure(void)
 {
     int status;
@@ -319,9 +396,7 @@ void stdin_configure(void)
     memcpy(&stdin_new, &stdin_old, sizeof(stdin_old));
 
     /* Reconfigure stdin (RAW configuration) */
-    stdin_new.c_iflag &= ~(ICRNL); // Do not translate CR -> NL on input
-    stdin_new.c_oflag &= ~(OPOST);
-    stdin_new.c_lflag &= ~(ECHO|ICANON|ISIG|ECHOE|ECHOK|ECHONL);
+    cfmakeraw(&stdin_new);
 
     /* Control characters */
     stdin_new.c_cc[VTIME] = 0; /* Inter-character timer unused */
@@ -339,9 +414,9 @@ void stdin_configure(void)
     atexit(&stdin_restore);
 }
 
-void stdin_restore(void)
+void stdout_restore(void)
 {
-    tcsetattr(STDIN_FILENO, TCSANOW, &stdin_old);
+    tcsetattr(STDOUT_FILENO, TCSANOW, &stdout_old);
 }
 
 void stdout_configure(void)
@@ -350,7 +425,7 @@ void stdout_configure(void)
 
     /* Disable line buffering in stdout. This is necessary if we
      * want things like local echo to work correctly. */
-    setbuf(stdout, NULL);
+    setvbuf(stdout, NULL, _IONBF, 0);
 
     /* Save current stdout settings */
     if (tcgetattr(STDOUT_FILENO, &stdout_old) < 0)
@@ -363,8 +438,7 @@ void stdout_configure(void)
     memcpy(&stdout_new, &stdout_old, sizeof(stdout_old));
 
     /* Reconfigure stdout (RAW configuration) */
-    stdout_new.c_oflag &= ~(OPOST);
-    stdout_new.c_lflag &= ~(ECHO|ICANON|ISIG|ECHOE|ECHOK|ECHONL);
+    cfmakeraw(&stdout_new);
 
     /* Control characters */
     stdout_new.c_cc[VTIME] = 0; /* Inter-character timer unused */
@@ -385,11 +459,6 @@ void stdout_configure(void)
     atexit(&stdout_restore);
 }
 
-void stdout_restore(void)
-{
-    tcsetattr(STDOUT_FILENO, TCSANOW, &stdout_old);
-}
-
 void tty_configure(void)
 {
     bool token_found = true;
@@ -514,7 +583,9 @@ void tty_configure(void)
         tio.c_cflag &= ~PARODD;
     }
     else if (strcmp("none", option.parity) == 0)
+    {
         tio.c_cflag &= ~PARENB;
+    }
     else
     {
         error_printf("Invalid parity");
@@ -535,26 +606,44 @@ void tty_configure(void)
     while (token_found == true)
     {
         if (token == NULL)
+        {
             token = strtok(buffer,",");
+        }
         else
+        {
             token = strtok(NULL, ",");
+        }
 
         if (token != NULL)
         {
             if (strcmp(token,"INLCR") == 0)
+            {
                 tio.c_iflag |= INLCR;
+            }
             else if (strcmp(token,"IGNCR") == 0)
+            {
                 tio.c_iflag |= IGNCR;
+            }
             else if (strcmp(token,"ICRNL") == 0)
+            {
                 tio.c_iflag |= ICRNL;
+            }
             else if (strcmp(token,"OCRNL") == 0)
+            {
                 map_o_cr_nl = true;
+            }
             else if (strcmp(token,"ODELBS") == 0)
+            {
                 map_o_del_bs = true;
+            }
             else if (strcmp(token,"INLCRNL") == 0)
+            {
                 map_i_nl_crnl = true;
+            }
             else if (strcmp(token, "ONLCRNL") == 0)
+            {
                 map_o_nl_crnl = true;
+            }
             else
             {
                 printf("Error: Unknown mapping flag %s\n", token);
@@ -562,7 +651,9 @@ void tty_configure(void)
             }
         }
         else
+        {
             token_found = false;
+        }
     }
     free(buffer);
 }
@@ -586,7 +677,8 @@ void tty_wait_for_device(void)
             tv.tv_sec = 0;
             tv.tv_usec = 1;
             first = false;
-        } else
+        }
+        else
         {
             /* Wait up to 1 second */
             tv.tv_sec = 1;
@@ -619,7 +711,8 @@ void tty_wait_for_device(void)
                 previous_char = input_char;
             }
             socket_handle_input(&rdfs, NULL);
-        } else if (status == -1)
+        }
+        else if (status == -1)
         {
             error_printf("select() failed (%s)", strerror(errno));
             exit(EXIT_FAILURE);
@@ -627,7 +720,8 @@ void tty_wait_for_device(void)
 
         /* Test for accessible device file */
         status = access(option.tty_device, R_OK);
-        if (status == 0) {
+        if (status == 0)
+        {
             last_errno = 0;
             return;
         }
@@ -656,7 +750,60 @@ void tty_restore(void)
     tcsetattr(fd, TCSANOW, &tio_old);
 
     if (connected)
+    {
         tty_disconnect();
+    }
+}
+
+void forward_to_tty(int fd, char output_char)
+{
+    int status;
+
+    /* Map output character */
+    if ((output_char == 127) && (map_o_del_bs))
+    {
+        output_char = '\b';
+    }
+    if ((output_char == '\r') && (map_o_cr_nl))
+    {
+        output_char = '\n';
+    }
+
+    /* Map newline character */
+    if ((output_char == '\n' || output_char == '\r') && (map_o_nl_crnl))
+    {
+        const char *crlf = "\r\n";
+
+        optional_local_echo(crlf[0]);
+        optional_local_echo(crlf[1]);
+        status = tty_write(fd, crlf, 2);
+        if (status < 0)
+        {
+            warning_printf("Could not write to tty device");
+        }
+
+        tx_total += 2;
+    }
+    else
+    {
+        if (print_mode == HEX)
+        {
+            output_hex(output_char);
+        }
+        else
+        {
+            /* Send output to tty device */
+            optional_local_echo(output_char);
+            status = tty_write(fd, &output_char, 1);
+            if (status < 0)
+            {
+                warning_printf("Could not write to tty device");
+            }
+
+            /* Update transmit statistics */
+            tx_total++;
+        }
+    }
 }
 
 int tty_connect(void)
@@ -664,18 +811,15 @@ int tty_connect(void)
     fd_set rdfs;           /* Read file descriptor set */
     int    maxfd;          /* Maximum file descriptor used */
     char   input_char, output_char;
+    char   input_buffer[BUFSIZ];
     static char previous_char = 0;
     static bool first = true;
     int    status;
-    bool next_timestamp = false;
+    bool   next_timestamp = false;
     char*  now = NULL;
 
     /* Open tty device */
-#ifdef __APPLE__
-    fd = open(option.tty_device, O_RDWR | O_NOCTTY | O_NONBLOCK );
-#else
-    fd = open(option.tty_device, O_RDWR | O_NOCTTY);
-#endif
+    fd = open(option.tty_device, O_RDWR | O_NOCTTY | O_NONBLOCK);
     if (fd < 0)
     {
         error_printf_silent("Could not open tty device (%s)", strerror(errno));
@@ -706,7 +850,9 @@ int tty_connect(void)
     print_tainted = false;
 
     if (option.timestamp)
+    {
         next_timestamp = true;
+    }
 
     /* Manage print output mode */
     if (option.hex_mode)
@@ -722,7 +868,9 @@ int tty_connect(void)
 
     /* Save current port settings */
     if (tcgetattr(fd, &tio_old) < 0)
+    {
         goto error_tcgetattr;
+    }
 
 #ifdef HAVE_IOSSIOSPEED
     if (!standard_baudrate)
@@ -787,12 +935,23 @@ int tty_connect(void)
             if (FD_ISSET(fd, &rdfs))
             {
                 /* Input from tty device ready */
-                if (read(fd, &input_char, 1) > 0)
+                ssize_t bytes_read = read(fd, input_buffer, BUFSIZ);
+                if (bytes_read <= 0)
                 {
-                    /* Update receive statistics */
-                    rx_total++;
+                    /* Error reading - device is likely unplugged */
+                    error_printf_silent("Could not read from tty device");
+                    goto error_read;
+                }
 
-                    /* Print timestamp on new line, if desired. */
+                /* Update receive statistics */
+                rx_total += bytes_read;
+
+                /* Process input byte by byte */
+                for (int i=0; i<bytes_read; i++)
+                {
+                    input_char = input_buffer[i];
+
+                    /* Print timestamp on new line if enabled */
                     if (next_timestamp && input_char != '\n' && input_char != '\r')
                     {
                         now = current_time();
@@ -801,14 +960,7 @@ int tty_connect(void)
                             ansi_printf_raw("[%s] ", now);
                             if (option.log)
                             {
-                                log_write('[');
-                                while (*now != '\0')
-                                {
-                                    log_write(*now);
-                                    ++now;
-                                }
-                                log_write(']');
-                                log_write(' ');
+                                log_printf("[%s] ", now);
                             }
                             next_timestamp = false;
                         }
@@ -820,117 +972,96 @@ int tty_connect(void)
                         print('\r');
                         print('\n');
                         if (option.timestamp)
+                        {
                             next_timestamp = true;
-                    } else
+                        }
+                    }
+                    else
                     {
                         /* Print received tty character to stdout */
                         print(input_char);
                     }
-                    fflush(stdout);
 
                     /* Write to log */
                     if (option.log)
-                        log_write(input_char);
+                    {
+                        log_putc(input_char);
+                    }
 
                     socket_write(input_char);
 
                     print_tainted = true;
 
                     if (input_char == '\n' && option.timestamp)
+                    {
                         next_timestamp = true;
-                } else
-                {
-                    /* Error reading - device is likely unplugged */
-                    error_printf_silent("Could not read from tty device");
-                    goto error_read;
+                    }
                 }
             }
-            if (FD_ISSET(STDIN_FILENO, &rdfs))
+            else if (FD_ISSET(STDIN_FILENO, &rdfs))
             {
-                forward = true;
-
                 /* Input from stdin ready */
-                status = read(STDIN_FILENO, &input_char, 1);
-                if (status <= 0)
+                ssize_t bytes_read = read(STDIN_FILENO, input_buffer, BUFSIZ);
+                if (bytes_read <= 0)
                 {
                     error_printf_silent("Could not read from stdin");
                     goto error_read;
                 }
 
-                /* Forward input to output */
-                output_char = input_char;
-
-                if (interactive_mode)
+                /* Process input byte by byte */
+                for (int i=0; i<bytes_read; i++)
                 {
-                    /* Do not forward ctrl-t key */
-                    if (input_char == KEY_CTRL_T)
-                        forward = false;
+                    input_char = input_buffer[i];
 
-                    /* Handle commands */
-                    handle_command_sequence(input_char, previous_char, &output_char, &forward);
+                    /* Forward input to output */
+                    output_char = input_char;
+                    forward = true;
 
-                    /* Save previous key */
-                    previous_char = input_char;
+                    if (interactive_mode)
+                    {
+                        /* Do not forward ctrl-t key */
+                        if (input_char == KEY_CTRL_T)
+                        {
+                            forward = false;
+                        }
+
+                        /* Handle commands */
+                        handle_command_sequence(input_char, previous_char, &output_char, &forward);
+
+                        /* Save previous key */
+                        previous_char = input_char;
+
+                        if (print_mode == HEX)
+                        {
+                            if (!is_valid_hex(input_char))
+                            {
+                                warning_printf("Invalid hex character: '%d' (0x%02x)", input_char, input_char);
+                                forward = false;
+                            }
+                        }
+                    }
+
+                    if (forward)
+                    {
+                        forward_to_tty(fd, output_char);
+                    }
                 }
+
+                tty_flush(fd);
             }
             else
             {
                 forward = socket_handle_input(&rdfs, &output_char);
-            }
 
-            if (forward)
-            {
-                if (print_mode == HEX)
+                if (forward)
                 {
-                    if (!is_valid_hex(input_char))
-                    {
-                        warning_printf("Invalid hex character: '%d' (0x%02x)", input_char, input_char);
-                        continue;
-                    }
+                    forward_to_tty(fd, output_char);
                 }
 
-                /* Map output character */
-                if ((output_char == 127) && (map_o_del_bs))
-                    output_char = '\b';
-                if ((output_char == '\r') && (map_o_cr_nl))
-                    output_char = '\n';
-
-                /* Map newline character */
-                if ((output_char == '\n' || output_char == '\r') && (map_o_nl_crnl)) {
-                    const char *crlf = "\r\n";
-
-                    optional_local_echo(crlf[0]);
-                    optional_local_echo(crlf[1]);
-                    status = write(fd, crlf, 2);
-                    if (status < 0)
-                        warning_printf("Could not write to tty device");
-
-                    tx_total += 2;
-                    delay(option.output_delay);
-                } else
-                {
-                    if (print_mode == HEX)
-                    {
-                        output_hex(output_char);
-                    }
-                    else
-                    {
-                        /* Send output to tty device */
-                        optional_local_echo(output_char);
-                        status = write(fd, &output_char, 1);
-                        if (status < 0)
-                            warning_printf("Could not write to tty device");
-                        fsync(fd);
-
-                        /* Update transmit statistics */
-                        tx_total++;
-                    }
-
-                    /* Insert output delay */
-                    delay(option.output_delay);
-                }
+                tty_flush(fd);
             }
-        } else if (status == -1)
+        }
+        else if (status == -1)
         {
             error_printf("Error: select() failed (%s)", strerror(errno));
             exit(EXIT_FAILURE);
